package com.laguerre.kan;

import java.io.InputStream;

public final class MeshImporter {
	public static enum ImportMaterialMode {
		BASED_ON_MATERIAL_NAME
	}

	public static enum ImportNormalTangentMode {
		IMPORT_FROM_FILE,
		CALCULATE,
		NONE
	}

	public static final class Settings {
		public float globalScale = 1.0f;
		public boolean swappedUV = false;
		public ImportMaterialMode materialImportMode = ImportMaterialMode.BASED_ON_MATERIAL_NAME;
		public ImportNormalTangentMode normalImportMode = ImportNormalTangentMode.IMPORT_FROM_FILE;
		public ImportNormalTangentMode tangentImportMode = ImportNormalTangentMode.IMPORT_FROM_FILE;
	}

	public static interface IMeshImporterImpl {
		public void setSettings(Settings settings);
		public void importToTarget(InputStream inputStream, SceneObject sobTarget);
	}

	private final Settings settings = new Settings();

	public <T extends IMeshImporterImpl> void importToTarget(
			InputStream inputStream,
			SceneObject sobTarget,
			Class<T> meshImporterImplType) {
		T meshImporterImpl = null;
		try {
			meshImporterImpl = meshImporterImplType.newInstance();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		if (meshImporterImpl != null) {
			meshImporterImpl.setSettings(this.settings);
			meshImporterImpl.importToTarget(inputStream, sobTarget);
		}
	}

	public void setGlobalScale(float globalScale) {
		this.settings.globalScale = globalScale;
	}

	public float getGlobalScale() {
		return this.settings.globalScale;
	}

	public void setSwappedUV(boolean swappedUV) {
		this.settings.swappedUV = swappedUV;
	}

	public boolean isSwappedUV() {
		return this.settings.swappedUV;
	}

	public void setMaterialImportMode(ImportMaterialMode materialImportMode) {
		this.settings.materialImportMode = materialImportMode;
	}

	public ImportMaterialMode getMaterialImportMode() {
		return this.settings.materialImportMode;
	}

	public void setNormalImportMode(ImportNormalTangentMode normalImportMode) {
		this.settings.normalImportMode = normalImportMode;
	}

	public ImportNormalTangentMode getNormalImportMode() {
		return this.settings.normalImportMode;
	}

	public void setTangentImportMode(ImportNormalTangentMode tangentImportMode) {
		this.settings.tangentImportMode = tangentImportMode;
	}

	public ImportNormalTangentMode getTangentImportMode() {
		return this.settings.tangentImportMode;
	}
}
