package com.laguerre.kan;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.nio.IntBuffer;

import android.graphics.Bitmap;
import android.opengl.GLES20;
import android.os.Environment;

public final class RenderTexture extends Texture {
	private int prevTexturehandle;
	private int width;
	private int height;
	private boolean swapped;

	public void init(int width, int height) {
		this.prevTexturehandle = -1;
		this.width = width;
		this.height = height;
		this.swapped = false;
	}

	@Override
	public int getWidth() {
		return this.width;
	}

	@Override
	public int getHeight() {
		return this.height;
	}

	@Override
	public void create() {
		// Makes a handle and an initialization for the prev texture.
		doCreation();
		this.prevTexturehandle = this.textureHandle[0];
		GLES20.glTexImage2D(GLES20.GL_TEXTURE_2D, 0, GLES20.GL_RGBA,
                this.width, this.height, 0,
                GLES20.GL_RGBA, GLES20.GL_UNSIGNED_BYTE,
                null);

		// Makes a handle for the texture that will be the current one + init.
		doCreation();
		GLES20.glTexImage2D(GLES20.GL_TEXTURE_2D, 0, GLES20.GL_RGBA,
                this.width, this.height, 0,
                GLES20.GL_RGBA, GLES20.GL_UNSIGNED_BYTE,
                null);
	}

	public int getPrevTextureHandle() {
		return this.prevTexturehandle;
	}

	@Override
	public void deleteTexture() {
		super.deleteTexture();
		if (this.prevTexturehandle > 0) {
			final int[] prevTexH = { this.prevTexturehandle };
			GLES20.glDeleteTextures(1, prevTexH, 0);
			this.prevTexturehandle = -1;
		}
	}

	public void swapPrevAndCurrentTexture() {
		final int h = this.textureHandle[0];
		this.textureHandle[0] = this.prevTexturehandle;
		this.prevTexturehandle = h;
		// At least one swap occurred.
		this.swapped = true;
	}

	public void saveCurrentFBToImage(String filename) {
		final IntBuffer targetBuffer = IntBuffer.allocate(width * height * 4);
		GLES20.glReadPixels(
				0, 0, this.width, this.height, GLES20.GL_RGBA,
				GLES20.GL_UNSIGNED_BYTE, targetBuffer);

		final String path = Environment.getExternalStorageDirectory().toString();
		final int[] pixels = targetBuffer.array();
		final Bitmap bmp = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
		bmp.setPixels(pixels, 0, width, 0, 0, width, height);

		try {
			final FileOutputStream out = new FileOutputStream(path + "/" + filename);
			bmp.compress(Bitmap.CompressFormat.PNG, 90, out);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}

	public boolean hasSwapped() {
		return this.swapped;
	}

	public void markAsNotSwapped() {
		this.swapped = false;
	}
}
