package com.laguerre.mytest;

import com.laguerre.kan.AManualMesh;

public class Cube extends AManualMesh {
	private static final float[] cubeVertices = {
		/*
		 *   4+------+ 7      ^ y
		 *   /      / |       |
		 *  0+-----+3 |       0---> x
		 *   |     |  +6     z
		 *   |     | /
		 *  1+-----+2
		 *
		 */

		// front: 0, 1, 2, 0, 2, 3
		//  => 0, 1, 2, 0, 2, 3
		-0.5f,  0.5f,  0.5f, 1.0f,   // 0 - 0
		-0.5f, -0.5f,  0.5f, 1.0f,   // 1 - 1
		 0.5f, -0.5f,  0.5f, 1.0f,   // 2 - 2
		 0.5f,  0.5f,  0.5f, 1.0f,   // 3 - 3

		// right: 3, 2, 6, 3, 6, 7
		//  => 4, 5, 6, 4, 6, 7
		 0.5f,  0.5f,  0.5f, 1.0f,   // 3 - 4
		 0.5f, -0.5f,  0.5f, 1.0f,   // 2 - 5
		 0.5f, -0.5f, -0.5f, 1.0f,   // 6 - 6
		 0.5f,  0.5f, -0.5f, 1.0f,   // 7 - 7

		// back: 7, 6, 5, 7, 5, 4
		//  => 8, 9, 10, 8, 10, 11
		 0.5f,  0.5f, -0.5f, 1.0f,   // 7 - 8
		 0.5f, -0.5f, -0.5f, 1.0f,   // 6 - 9
		-0.5f, -0.5f, -0.5f, 1.0f,   // 5 - 10
		-0.5f,  0.5f, -0.5f, 1.0f,   // 4 - 11

		// left: 4, 5, 1, 4, 1, 0
		//  => 12, 13, 14, 12, 14, 15
		-0.5f,  0.5f, -0.5f, 1.0f,   // 4 - 12
		-0.5f, -0.5f, -0.5f, 1.0f,   // 5 - 13
		-0.5f, -0.5f,  0.5f, 1.0f,   // 1 - 14
		-0.5f,  0.5f,  0.5f, 1.0f,   // 0 - 15

		// top: 4, 0, 3, 4, 3, 7
		//  => 16, 17, 18, 16, 18, 19
		-0.5f,  0.5f, -0.5f, 1.0f,   // 4 - 16
		-0.5f,  0.5f,  0.5f, 1.0f,   // 0 - 17
		 0.5f,  0.5f,  0.5f, 1.0f,   // 3 - 18
		 0.5f,  0.5f, -0.5f, 1.0f,   // 7 - 19

		// bottom: 1, 5, 6, 1, 6, 2
		//  => 20, 21, 22, 20, 22, 23
		-0.5f, -0.5f,  0.5f, 1.0f,   // 1 - 20
		-0.5f, -0.5f, -0.5f, 1.0f,   // 5 - 21
		 0.5f, -0.5f, -0.5f, 1.0f,   // 6 - 22
		 0.5f, -0.5f,  0.5f, 1.0f,   // 2 - 23
	};

	private static final float[] cubeNormals = {
		// front
		 0f,  0f,  1f,    0f,  0f,  1f,    0f,  0f,  1f,    0f,  0f,  1f,
		// right
		 1f,  0f,  0f,    1f,  0f,  0f,    1f,  0f,  0f,    1f,  0f,  0f,
		// back
		 0f,  0f, -1f,    0f,  0f, -1f,    0f,  0f, -1f,    0f,  0f, -1f,
		// left
		-1f,  0f,  0f,   -1f,  0f,  0f,   -1f,  0f,  0f,   -1f,  0f,  0f,
		// top
		 0f,  1f,  0f,    0f,  1f,  0f,    0f,  1f,  0f,    0f,  1f,  0f,
		// bottom
		 0f, -1f,  0f,    0f, -1f,  0f,    0f, -1f,  0f,    0f, -1f,  0f
	};

	private static final float[] cubeTangents = {
		// front
		 1f, 0f,  0f, 1f,   1f, 0f,  0f, 1f,   1f, 0f,  0f, 1f,   1f, 0f,  0f, 1f,
		// right
		 0f, 0f, -1f, 1f,   0f, 0f, -1f, 1f,   0f, 0f, -1f, 1f,   0f, 0f, -1f, 1f,
		// back
		-1f, 0f,  0f, 1f,  -1f, 0f,  0f, 1f,  -1f, 0f,  0f, 1f,  -1f, 0f,  0f, 1f,
		// left
		 0f, 0f,  1f, 1f,   0f, 0f,  1f, 1f,   0f, 0f,  1f, 1f,   0f, 0f,  1f, 1f,
		// top
		 1f, 0f,  0f, 1f,   1f, 0f,  0f, 1f,   1f, 0f,  0f, 1f,   1f, 0f,  0f, 1f,
		// bottom
		-1f, 0f,  0f, 1f,  -1f, 0f,  0f, 1f,  -1f, 0f,  0f, 1f,  -1f, 0f,  0f, 1f
	};

	private static final float[] cubeUVs = {
		/*
		 *  Faces in OpenGL texture:
		 *
		 *  +---+---+---+
		 *  |fr |ri |ba +
		 *  +---+---+---+
		 *  |le |to |bo |
		 *  +---+---+---+
		 *
		 */
		// front
		0.0f, 1.0f,       0.0f, 0.5f,       0.333333f, 0.5f,   0.333333f, 1.0f,
		// right
		0.333333f, 1.0f,  0.333333f, 0.5f,  0.666666f, 0.5f,   0.666666f, 1.0f,
		// back
		0.666666f, 1.0f,  0.666666f, 1.0f,  1.0f, 0.5f,        1.0f, 1.0f,
		// left
		0.0f, 0.5f,       0.0f, 0.0f,       0.333333f, 0.0f,   0.333333f, 0.5f,
		// top
		0.333333f, 0.5f,  0.333333f, 0.0f,  0.666666f, 0.0f,   0.666666f, 0.5f,
		// bottom
		0.666666f, 0.5f,  0.666666f, 0.5f,  1.0f, 0.0f,        1.0f, 0.5f
	};

	private static short[] drawList = {
		// front
		0, 1, 2, 0, 2, 3,
		// right
		4, 5, 6, 4, 6, 7,
		// back
		8, 9, 10, 8, 10, 11,
		// left
		12, 13, 14, 12, 14, 15,
		// top
		16, 17, 18, 16, 18, 19,
		// bottom
		20, 21, 22, 20, 22, 23
	};

	@Override
	public void init() {
		setVertices(cubeVertices, 0);
		setNormals(cubeNormals, 0);
		setTangents(cubeTangents, 0);
		setUVs(cubeUVs, 0);
		setTriangles(drawList, 0);
		build();
	}
}
